static char rcsid[] = "$Id: qonfrig.c,v 1.1 1996/12/25 18:02:44 jrh Exp $";

/*-------------------------------------------------------------------.
|   QONFRIG : A program to save and restore QJUMP Level 1 config     |
|   blocks.  This program is designed for use with QTPI and thus     |
|   only handles the config types used by QTPI. The program is used  |
|   in two phases.						     |
| 								     |
|   With a single argument, QONFRIG will record the config items     |
|   used by the program to stdout. This may be saved using the	     |
|   redirection operators.					     |
| 								     |
|   ex qonfrig;'qtpi_160 >qtpi_config' : REM save config options for |
|   v1.60							     |
| 								     |
|   With two arguments, the program defined by the first argument is |
|   Configed using the config values saved by a one argument mode    |
|   qonfrig.							     |
| 								     |
|   So, for a hypothetical qtpi_161, the config values saved by the  |
|   above command could be applied to the new version.		     |
| 								     |
|   ex qonfrig;'qtpi_161 qtpi_config' : REM apply saved config	     |
| 								     |
| The format of the saved config data:				     |
| 								     |
| type_code desc_len desc data...				     |
| 								     |
| The data is the hex value for numeric or a length and charcaters   |
| for text (with a space between).				     |
| 								     |
|   (c) Jonathan Hudson 11/1996. Free and freely distributable.	     |
`-------------------------------------------------------------------*/

/*
 * $Log: qonfrig.c,v $
 * Revision 1.1  1996/12/25 18:02:44  jrh
 * Initial revision
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#ifdef __i386__
# define SHORTVAL(x) swapword(x)
# define LONGVAL(x)  swaplong(x)
 ushort inline swapword (ushort val)
 {
    return (ushort) (val << 8) + (val >> 8);
 }

 ulong inline swaplong (ulong val)
 {
    return (ulong) (((ulong) swapword (val & 0xFFFF) << 16) |
		    (ulong) swapword (val >> 16));
 }
#else
# define SHORTVAL(x) (x)
# define LONGVAL(x)  (x)
char _prog_name[] = "qonfrig v0.00";
#include <qdos.h>
#endif

short qprintstr(short *p)
{
    short m;
    m = SHORTVAL (*p);
    printf ("%d %.*s", m, m, (char*) (p + 1));    
    return m;
}

void CheckFile (short *p, long size)
{
    short m, n;
    short *x = (short *)((char *)p + size);
    
    for (; p < x; p++)
    {
	if (*p == 0x3c3c)
	{
	    if (memcmp ((p + 1), "QCFX>>01", 8) == 0)
	    {
		m = SHORTVAL (*(p+5));
		m = (m + 1) / 2;
		n = SHORTVAL (*(p+6+m));

		for(p = p + 7 + m + (n+1)/2; *p != -1 ; p += 6)
		{
		    short *desc;
		    void *item;
		    char typ = *(char *)p;
		    
		    desc = p + 4 + SHORTVAL(*(p + 4))/2;
		    item = p + 1 + SHORTVAL(*(p + 1))/2;
		    
		    printf("%d ", typ);
		    
		    qprintstr(desc);
		    putchar (' ');
		    
		    switch(typ)
		    {
		      case 0:
			qprintstr((short *)item+1);
			break;
		      case 4:
			printf ("%02x", *(unsigned char *)item);
			break;
		      case 10:
			printf ("%04x", SHORTVAL(*(short *)item));
			break;
		    }
		    putchar('\n');
		}
		break;
	    }
	}
    }
}

void Frigit(char *line, short *p, short *res)
{
    char ptyp;
    char *q, *qlin;
    
    ptyp = strtol(line, &qlin, 10);

    if(ptyp == 0 || ptyp == 4 || ptyp == 10)
    {
	qlin++;
	for(; *p != -1 ; p += 6)
	{
	    short *desc;
	    void *item;
	    short n;
	    char typ = *(char *)p;
	    q = qlin;
	    
	    if(typ == ptyp)
	    {
		desc = p + 4 + SHORTVAL(*(p + 4))/2;
		n = strtol(q, &q, 10);
		
		if(n == SHORTVAL(*desc))
		{
		    q++;
		    if(memcmp(q, (char *)(desc+1), n) == 0)
		    {
			q += n;
			*res = *res + 1;
			item = p + 1 + SHORTVAL(*(p + 1))/2;
			switch(typ)
			{
			  case 0:
			    {
				short nn, *out;
				out = (short *)item + 1;
				nn = strtol(q, &q, 10);
				q++;
				*out = SHORTVAL(nn); 
				memcpy((char *)(out+1), q, nn);
				printf("Set %.*s=%.*s\n", n, (char *)(desc+1), 
				       nn, q);
			    }
			    break;
			  case 4:
			    {
				unsigned char *out;
				out = item;
				*out = strtoul(q, NULL, 16);
				printf("Set %.*s=%x\n", n, (char *)(desc+1), 
				       *out);
			    }				
			    break;
			  case 10:
			    {
				unsigned short *out, val;
				out = item;
				val = strtoul(q, NULL, 16);
				*out = SHORTVAL(val);
				printf("Set %.*s=%x\n", n, (char *)(desc+1), 
				       val);
			    }
			    break;
			}
			break;
		    }
		}
	    }
	}
    }
}

short PatchFile (short *p, long size, char *file)
{
    short m, n;
    short *x = (short *)((char *)p + size);
    short *qfg;
    short patched = 0;
    FILE *fp;
    
    for (; p < x; p++)
    {
	if (*p == 0x3c3c)
	{
	    if (memcmp ((p + 1), "QCFX>>01", 8) == 0)
	    {
		m = SHORTVAL (*(p+5));
		m = (m + 1) / 2;
		n = SHORTVAL (*(p+6+m));
		qfg = p + 7 + m + (n+1)/2;
		if((fp = fopen(file, "r")))
		{
		    char buf[128];
		    
		    while(fgets(buf, sizeof(buf), fp))
		    {
			if(isdigit(*buf))
			{
			    Frigit(buf, qfg, &patched);
			}
		    }
		    fclose(fp);
		}
	    }
	}
    }
    return patched;
}

int main (int ac, char **av)
{
    struct stat s;
    int fd;
    void *m;
    
    if(ac == 2 || ac == 3)
    {
	fd = open (*(av+1), (ac == 2) ? O_RDONLY : O_RDWR, 0);
	if (fd != 0)
	{
	    fstat (fd,  &s);
	    
	    if ((m = malloc ((size_t)s.st_size)))
	    {
		read (fd, m, s.st_size);
		if(ac == 2)
		{
		    CheckFile(m, s.st_size);
		}
		else
		{
		    if(PatchFile(m, s.st_size, *(av+2)))
		    {
			lseek(fd, 0, SEEK_SET);
			write (fd, m, s.st_size);
		    }			
		}
		free (m);
	    }
	    close (fd);
	}
    }
    else
    {
	fputs("qonfrig prog [settings_file]\n", stderr);
    }
    
    return 0;
}
